package com.ann.demo.interfaces.impl;

import com.ann.demo.entity.constant.AliasName;
import com.ann.demo.entity.constant.Identification;
import com.ann.demo.entity.constant.InterfaceName;
import com.ann.demo.entity.filing.EMRDictionary;
import com.ann.demo.entity.filing.MessageLog;
import com.ann.demo.normalEntity.*;
import com.ann.demo.interfaces.HomepageDictionary;
import com.ann.demo.service.*;
import com.ann.demo.utils.XMLUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;

import javax.jws.WebService;
import java.util.Date;

/**
 * @Author: LeiJiaXin
 * @Date: 2019/6/11 17:01
 */

@WebService(serviceName = "HomepageDictionary", // 与接口中指定的name一致
        targetNamespace = "http://interfaces.demo.ann.com/", // 与接口中的命名空间一致,一般是接口的包名倒
        endpointInterface = "com.ann.demo.interfaces.HomepageDictionary"// 接口地址
)
public class HomepageDictionaryImpl implements HomepageDictionary {

    private static final Logger logger = LoggerFactory.getLogger(HomepageDictionaryImpl.class);

    @Autowired
    private MessageService messageService;

    @Autowired
    private UserService userService;

    @Autowired
    private EMRDictionaryService emrDictionaryService;

    @Override
    public String ReceiveDict(String xmlStr) {
        String outContent = "失败";
        try {
            //先保存变量
            long startTime = System.currentTimeMillis();
            MessageLog messageLog = new MessageLog(xmlStr, InterfaceName.RECEIVE_DICT, new Date());
            messageService.save(messageLog);
            // 1、定义变量
            Integer result = 1;
            boolean flag = false;
            // 2、 将XML转换成对象
            InputPara inputPara = XMLUtils.xmlStrToObject(xmlStr, InputPara.class);
            if (inputPara != null) {
                // 3、根据IDENTIFICATION 进行相应的操作
                String identification = inputPara.getIDENTIFICATION();
                if (inputPara.getALIASNAME().equals(AliasName.USER)) {
                    User user = XMLUtils.xmlStrToObject(inputPara.getVALUE(), User.class);
                    //将user转换
                    if (user != null) {
                        if (identification != null) {
                            switch (identification) {
                                case Identification.DELETE:
                                    userService.delete(user);
                                    break;
                                case Identification.INSERT:
                                    userService.save(user);
                                    break;
                                case Identification.UPDATE:
                                    userService.save(user);
                                    break;
                            }
                            messageLog.setType(AliasName.USER);
                            flag = true;
                        }
                    }
                } else if (inputPara.getALIASNAME().equals(AliasName.DEPARTMENT)) {
                    EMRDictionary emrDictionary = XMLUtils.xmlStrToObject(inputPara.getVALUE(), EMRDictionary.class);
                    if (emrDictionary != null) {
                        // 3、根据IDENTIFICATION 进行相应的操作
                        if (identification != null) {
                            if (identification != null) {
                                switch (identification) {
                                    case Identification.DELETE:
                                        emrDictionaryService.delete(emrDictionary);
                                        break;
                                    case Identification.INSERT:
                                        emrDictionaryService.save(emrDictionary);
                                        break;
                                    case Identification.UPDATE:
                                        emrDictionaryService.save(emrDictionary);
                                        break;
                                }
                                messageLog.setType(AliasName.DEPARTMENT);
                                flag = true;
                            }
                        }
                    }
                }
            }

            // 4、判断是否成功  失败 的原因 我们插入报错 然后无法解析xml 传入参数为null
            outContent = XMLUtils.getReturnResult(flag);
            if (!flag) {
                result = 0;
            }
            //插入日志表
            long endTime = System.currentTimeMillis();
            messageLog.setValue1(outContent, result, (endTime - startTime), new Date());
            messageService.save(messageLog);

            logger.info("-----------------------用户、科室字典总用时长" + (endTime - startTime) + "-------------------------");
        }catch (Exception e){
            try {
                logger.error("出错咯！错误信息为{}，以及错误行数为:{}" ,e,e.getStackTrace()[0]);
                return XMLUtils.getReturnResult(false);
            } catch (Exception e1) {
                logger.error("出错咯！错误信息为{}，以及错误行数为:{}" ,e1,e1.getStackTrace()[0]);
            }
        }
        return outContent;
    }

}
