package com.ann.demo.service.impl;

import com.ann.demo.entity.filing.EMRDictionary;
import com.ann.demo.normalEntity.PowerDept;
import com.ann.demo.repository.EMRDictionaryRepository;
import com.ann.demo.primaryRepository.PowerDeptRepository;
import com.ann.demo.service.EMRDictionaryService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 * @Author: LeiJiaXin
 * @Date: 2019/9/21 16:08
 */
@Service
public class EMRDictionaryServiceImpl implements EMRDictionaryService {

    static final Logger logger = LoggerFactory.getLogger(EMRDictionaryServiceImpl.class);

    // distId
    public static Integer distId;

    @Value("${distId}")
    public void setDistId(Integer distId) {
        EMRDictionaryServiceImpl.distId = distId;
    }

    public static Integer getDistId() {
        return distId;
    }


    @Autowired
    EMRDictionaryRepository emrDictionaryRepository;

    @Autowired
    PowerDeptRepository powerDeptRepository;

    @Override
    public void save(EMRDictionary emrDictionary)  throws Exception {
        //先判断 字典表科室是否存在
        EMRDictionary emrDir = emrDictionaryRepository.findEMRDictionaryByCodeAndIsValid(emrDictionary.getDEPT_CODE(), 1);
        if( emrDir != null){
            emrDictionary.setId(emrDir.getId());
        }
        //保存emrDictionary
        emrDictionaryRepository.save(emrDictionary);

        //先判断 权限字典表科室是否存在
        PowerDept dept = powerDeptRepository.findByDeptCodeAndEffective(emrDictionary.getDEPT_CODE(), 1);
        PowerDept powerDept = new PowerDept(emrDictionary.getDEPT_NAME(),distId,emrDictionary.getDEPT_CODE());
        if(dept != null){
            powerDept.setDeptId(dept.getDeptId());
            powerDept.setUpdateDate(new Date().toLocaleString());
        }
        //保存power_dept一份
        powerDeptRepository.save(powerDept);
    }

    @Override
    public void delete(EMRDictionary emrDictionary) throws Exception {
        //打日志信息
        EMRDictionary emrDir = emrDictionaryRepository.findEMRDictionaryByCodeAndIsValid(emrDictionary.getDEPT_CODE(), 1);
        emrDictionaryRepository.updateEMRDictionary(emrDir.getId());
        PowerDept powerDept = powerDeptRepository.findByDeptCodeAndEffective(emrDictionary.getDEPT_CODE(), 1);
        powerDeptRepository.updatePowerDept(powerDept.getDeptId());
    }

    @Override
    public EMRDictionary findEMRDictionary(String code) {
        // ------------------确认 isValid 1是正常 0是停用
        return emrDictionaryRepository.findEMRDictionaryByCodeAndIsValid(code, 1);
    }
}
