package com.ann.demo.utils;

import com.ann.demo.normalEntity.OutPara;
import org.dom4j.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.beans.PropertyDescriptor;
import java.lang.reflect.Field;
import java.lang.reflect.Type;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @Description
 * @Date 2019/6/17 15:14
 * @Created by  ljx
 */
@Component
public class XMLUtils {

    static final Logger logger = LoggerFactory.getLogger(XMLUtils.class);

    private static SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyyMMddhhmmss");
    // 病案首页独享
    private static SimpleDateFormat simpleDateFormat1 = new SimpleDateFormat("yyyy/MM/dd hh:mm:ss");

    /**
     * xml字符串转成对象
     */
    public static <T> T xmlStrToObject(String xmlStr, Class<T> type) throws  Exception{
        if (StringUtils.isEmpty(xmlStr)) {
            return null;
        }
        Document doc = null; // 将xml格式的字符串转换成Document对象
        T obj = null; // 创建 JavaBean 对象
        obj = type.newInstance();
        doc = DocumentHelper.parseText(xmlStr);
        // 获取根节点
        Element root = doc.getRootElement();
        // 获取根节点下的所有元素
        List children = root.elements();
        // 循环所有子元素
        if (children != null && children.size() > 0) {
            for (int i = 0; i < children.size(); i++) {
                Element child = (Element) children.get(i);
                Field[] fields = obj.getClass().getDeclaredFields();
                for (Field field : fields) {
                    field.setAccessible(true);
                    String fieldName = field.getName();
                    if (fieldName.equals(child.getName())) {
                        Type genericType = field.getGenericType();
                        Object[] args = new Object[1];
                        String value = child.getTextTrim();

                        // 赋值
                        PropertyDescriptor descriptor = new PropertyDescriptor(fieldName, type);
                        if (("java.lang.Integer").equals(genericType.getTypeName()) || genericType.getTypeName().equals("int")) {
                            if (value != null && value.length() > 0) {
                                args[0] = Integer.parseInt(value + "");
                                descriptor.getWriteMethod().invoke(obj, args);
                            }
                        } else if ((Date.class.getName()).equals(genericType.getTypeName())) {
                            descriptor.getWriteMethod().invoke(obj, simpleDateFormat.parse(value, new ParsePosition(0)));
                        } else if (("java.lang.Long").equalsIgnoreCase(genericType.getTypeName()) || genericType.getTypeName().equals("long")) {
                            if (value != null && value.length() > 0) {
                                args[0] = Long.parseLong(value + "");
                                descriptor.getWriteMethod().invoke(obj, args);
                            }
                        } else if (("java.lang.Boolean").equalsIgnoreCase(genericType.getTypeName()) || genericType.getTypeName().equals("boolean")) {
                            if (value != null && value.length() > 0) {
                                args[0] = Boolean.valueOf(value + "");
                                descriptor.getWriteMethod().invoke(obj, args);
                            }
                        } else if (("java.lang.Double").equalsIgnoreCase(genericType.getTypeName()) || genericType.getTypeName().equals("double")) {
                            if (value != null && value.length() > 0) {
                                args[0] = Double.parseDouble(value + "");
                                descriptor.getWriteMethod().invoke(obj, args);
                            }
                        } else if (("java.lang.Float").equalsIgnoreCase(genericType.getTypeName()) || genericType.getTypeName().equals("float")) {
                            if (value != null && value.length() > 0) {
                                args[0] = Float.parseFloat(value + "");
                                descriptor.getWriteMethod().invoke(obj, args);
                            }
                        } else {
                            descriptor.getWriteMethod().invoke(obj, value);
                        }
                        break;
                    }
                }
            }
        }
        return obj;
    }

    public static String getReturnResult(boolean resultCode) {
        OutPara outPara = new OutPara();
        if (!resultCode) {
            outPara.setRESULT_CODE(false);
            outPara.setRESULT_CONTENT("失败");
        } else {
            outPara.setRESULT_CODE(true);
            outPara.setRESULT_CONTENT("成功");
        }
        return XMLUtils.generateXmlStr(outPara);
    }

    /**
     * 生成普通xml字符串
     */
    public static String generateXmlStr(OutPara outPara) {
        try {
            Field[] fields = outPara.getClass().getDeclaredFields();
            // 创建XML
            Document document = DocumentHelper.createDocument();
            Element response = document.addElement("RESPONSE");
            for (Field field : fields) {
                field.setAccessible(true);
                response
                        .addElement(field.getName())
                        .addText(field.get(outPara).toString());
            }
            return document.getRootElement().asXML();
        } catch (IllegalAccessException e) {
            logger.error("出错咯！生成普通xml字符串错误信息:" + e);
        }
        return null;
    }
}